from typing import Self
from dataclasses import dataclass

from ..register._byte import ByteRegister


@dataclass
class REG_I2C_CONTROL(ByteRegister):
    ADDRESS = 0x11

    crsd: bool = False
    frd: bool = False
    spe: bool = False
    wpd: bool = False
    si: bool = False
    bopga: bool = False
    ts: bool = False
    bgpcp: bool = False

    def to_byte(self) -> int:
        value = 0
        value |= int(self.crsd) << 7
        value |= int(self.frd) << 6
        value |= int(self.spe) << 5
        value |= int(self.wpd) << 4
        value |= int(self.si) << 3
        value |= int(self.bopga) << 2
        value |= int(self.ts) << 1
        value |= int(self.bgpcp) << 0
        return value

    @classmethod
    def from_byte(cls, byte: int) -> Self:
        return cls(
            crsd=bool(byte & (1 << 7)),
            frd=bool(byte & (1 << 6)),
            spe=bool(byte & (1 << 5)),
            wpd=bool(byte & (1 << 4)),
            si=bool(byte & (1 << 3)),
            bopga=bool(byte & (1 << 2)),
            ts=bool(byte & (1 << 1)),
            bgpcp=bool(byte & (1 << 0)),
        )



import time
from dataclasses import dataclass

try:
    import click
except ImportError:
    raise RuntimeError("Install with: pip install nau7802[cli]")

import smbus2

from . import NAU7802


@dataclass
class CLIContext:
    device: NAU7802


pass_context = click.make_pass_decorator(CLIContext)


@click.group(context_settings={"help_option_names": ["-h", "--help"]})
@click.version_option()
@click.option("--bus", default=1, show_default=True, type=int)
@click.pass_context
def main(ctx: click.Context, bus: int) -> None:
    """NAU7802 CLI tool."""
    ctx.obj = CLIContext(
        device=NAU7802(smbus2.SMBus(bus)),
    )


@main.command()
@pass_context
@click.option("--gain", "-g", default=7, show_default=True)
@click.option("--crs", "-c", default=3, show_default=True)
def init(ctx: CLIContext, gain: int, crs: int) -> None:
    ctx.device.initialize()
    ctx.device.set_gain(gain)
    ctx.device.set_crs(crs)


@main.command()
@pass_context
def standby(ctx: CLIContext) -> None:
    ctx.device.standby()


@main.command()
@pass_context
def resume(ctx: CLIContext) -> None:
    ctx.device.resume()


@main.command()
@pass_context
@click.argument("channel", type=int)
def set_channel(ctx: CLIContext, channel: int) -> None:
    if channel == 1:
        ctx.device.set_channel(1)
    elif channel == 2:
        ctx.device.set_channel(2)
    else:
        click.echo("Invalid channel")


@main.command()
@pass_context
@click.argument("ocal1", type=int)
def set_ocal1(ctx: CLIContext, ocal1: int) -> None:
    ctx.device.ocal1 = ocal1


@main.command()
@pass_context
@click.argument("ocal2", type=int)
def set_ocal2(ctx: CLIContext, ocal2: int) -> None:
    ctx.device.ocal2 = ocal2


@main.command()
@pass_context
@click.argument("calmod", type=int, default=0)
def calibrate(ctx: CLIContext, calmod: int) -> None:
    ctx.device.set_calmod(calmod)
    ctx.device.set_cals(True)

    timeout = time.time() + 1

    while ctx.device.cals:
        print("calibrating")
        time.sleep(0.05)
        if time.time() > timeout:
            print("timed out")

    if ctx.device.cal_err:
        print("error")
    else:
        print("success")


@main.command()
@pass_context
def read(ctx: CLIContext) -> None:
    print(ctx.device.adco.value)


@main.command()
@pass_context
def dump(ctx: CLIContext) -> None:
    print("NAU7802")
    print("  PU_CTRL     ", ctx.device.pu_ctrl)
    print("  CTRL1       ", ctx.device.ctrl1)
    print("  CTRL2       ", ctx.device.ctrl2)
    print("  OCAL1       ", ctx.device.ocal1.value)
    print("  GCAL1       ", hex(ctx.device.gcal1.value))
    print("  OCAL2       ", ctx.device.ocal2.value)
    print("  GCAL2       ", hex(ctx.device.gcal2.value))
    print("  I2C_CONTROL ", ctx.device.i2c_control)
    print("  ADCO        ", ctx.device.adco.value)
    print("  ADC_CTRL1   ", ctx.device.adc_ctrl1)
    print("  ADC_CTRL3   ", ctx.device.adc_ctrl3)
    print("  PWR_CTRL    ", ctx.device.pwr_ctrl)
    print("  REV_ID      ", ctx.device.rev_id)



from typing import Self
from dataclasses import dataclass

from ..register._byte import ByteRegister


@dataclass
class REG_CTRL2(ByteRegister):
    ADDRESS = 0x02

    chs: bool = False
    crs: int = 0
    cal_err: bool = False
    cals: bool = False
    calmod: int = 0

    def __post_init__(self) -> None:
        if not 0 <= self.crs <= 7:
            raise ValueError("crs out of range")
        if not 0 <= self.calmod <= 3:
            raise ValueError("calmod out of range")

    def to_byte(self) -> int:
        value = 0
        value |= int(self.chs) << 7
        value |= (self.crs & 0b111) << 4
        value |= int(self.cal_err) << 3
        value |= int(self.cals) << 2
        value |= self.calmod & 0b11
        return value

    @classmethod
    def from_byte(cls, byte: int) -> Self:
        crs = (byte >> 4) & 0b111
        calmod = byte & 0b11

        return cls(
            chs=bool(byte & (1 << 7)),
            crs=crs,
            cal_err=bool(byte & (1 << 3)),
            cals=bool(byte & (1 << 2)),
            calmod=calmod,
        )



from typing import Self
from dataclasses import dataclass

from ..register._byte import ByteRegister


@dataclass
class REG_REV_ID(ByteRegister):
    ADDRESS = 0x1F

    rev_id: int = 0
    res: int = 0

    def __post_init__(self) -> None:
        if not 0 <= self.rev_id <= 15:
            raise ValueError("rev_id out of range")
        if not 0 <= self.res <= 15:
            raise ValueError("res out of range")

    def to_byte(self) -> int:
        value = 0
        value |= (self.res & 0b1111) << 4
        value |= self.rev_id & 0b1111
        return value

    @classmethod
    def from_byte(cls, byte: int) -> Self:
        res = (byte >> 4) & 0b1111
        rev_id = byte & 0b1111

        return cls(
            res=res,
            rev_id=rev_id,
        )



from .adc_ctrl1 import REG_ADC_CTRL1
from .adc_ctrl3 import REG_ADC_CTRL3
from .adco import REG_ADCO
from .ctrl1 import REG_CTRL1
from .ctrl2 import REG_CTRL2
from .gcal1 import REG_GCAL1
from .gcal2 import REG_GCAL2
from .i2c_control import REG_I2C_CONTROL
from .ocal1 import REG_OCAL1
from .ocal2 import REG_OCAL2
from .pu_ctl import REG_PU_CTRL
from .pwr_ctrl import REG_PWR_CTRL
from .rev_id import REG_REV_ID

__all__ = [
    "REG_ADC_CTRL1",
    "REG_ADC_CTRL3",
    "REG_ADCO",
    "REG_CTRL1",
    "REG_CTRL2",
    "REG_GCAL1",
    "REG_GCAL2",
    "REG_I2C_CONTROL",
    "REG_OCAL1",
    "REG_OCAL2",
    "REG_PU_CTRL",
    "REG_PWR_CTRL",
    "REG_REV_ID",
]



from typing import Type, Self
from dataclasses import dataclass

from ._base import Register


@dataclass
class Int24Register(Register):
    WIDTH = 3

    value: int = 0  # signed 24-bit

    def __post_init__(self) -> None:
        if not (-(2**23) <= self.value < 2**23):
            raise ValueError("Value out of signed 24-bit range")

    def to_bytes(self) -> bytes:
        v = self.value & 0xFFFFFF
        return bytes(
            [
                (v >> 16) & 0xFF,
                (v >> 8) & 0xFF,
                v & 0xFF,
            ]
        )

    @classmethod
    def from_bytes(cls, data: bytes) -> Self:
        cls._assert_data_width(data)
        val = (data[0] << 16) | (data[1] << 8) | data[2]

        # sign extend
        if val & 0x800000:
            val -= 1 << 24

        return cls(value=val)



from typing import Self
from dataclasses import dataclass

from ..register._byte import ByteRegister


@dataclass
class REG_ADC_CTRL3(ByteRegister):
    ADDRESS = 0x1B

    rd_otp_sel: bool = False
    ldomode: bool = False
    pga_buff: bool = False
    pga_bp: bool = False
    pgainv: bool = False
    res2: bool = False
    res1: bool = False
    pgachpdis: bool = False

    def to_byte(self) -> int:
        value = 0
        value |= int(self.rd_otp_sel) << 7
        value |= int(self.ldomode) << 6
        value |= int(self.pga_buff) << 5
        value |= int(self.pga_bp) << 4
        value |= int(self.pgainv) << 3
        value |= int(self.res2) << 2
        value |= int(self.res1) << 1
        value |= int(self.pgachpdis) << 0
        return value

    @classmethod
    def from_byte(cls, byte: int) -> Self:
        return cls(
            rd_otp_sel=bool(byte & (1 << 7)),
            ldomode=bool(byte & (1 << 6)),
            pga_buff=bool(byte & (1 << 5)),
            pga_bp=bool(byte & (1 << 4)),
            pgainv=bool(byte & (1 << 3)),
            res2=bool(byte & (1 << 2)),
            res1=bool(byte & (1 << 1)),
            pgachpdis=bool(byte & (1 << 0)),
        )



from typing import Self
from dataclasses import dataclass

from ..register._byte import ByteRegister


@dataclass
class REG_CTRL1(ByteRegister):
    ADDRESS = 0x01

    drdyp: bool = False
    drdy_sel: bool = False
    vldo: int = 0
    pga: int = 1

    def __post_init__(self) -> None:
        if not 0 <= self.vldo <= 7:
            raise ValueError("vldo out of range")
        if not 0 <= self.pga <= 7:
            raise ValueError("pga out of range")

    def to_byte(self) -> int:
        value = 0
        value |= int(self.drdyp) << 7
        value |= int(self.drdy_sel) << 6
        value |= (self.vldo & 0b111) << 3
        value |= self.pga & 0b111
        return value

    @classmethod
    def from_byte(cls, byte: int) -> Self:
        pga = byte & 0b111
        vldo = (byte >> 3) & 0b111

        return cls(
            pga=pga,
            vldo=vldo,
            drdy_sel=bool(byte & (1 << 6)),
            drdyp=bool(byte & (1 << 7)),
        )
